;												
; SPI-Analyser-Sniffer fr ATmega8				
; Bernhard.Erfurt@gmx.de						
; 03/2016										
;												
;												
; Anschlussbelegung PINS						
;												
; PB0 ==> 										
; PB1 ==> 										
; PB2 ==> SPI-CS (EINGANG)						
; PB3 ==> SPI-MOSI								
; PB4 ==> SPI-MISO								
; PB5 ==> SPI-SCK								
; PB6 ==> 										
; PB7 ==> 										
;												
; PC0 ==> 										
; PC1 ==> 										
; PC2 ==> 										
; PC3 ==> 										
; PC4 ==> 										
; PC5 ==> 										
;												
; PD0 ==> 										
; PD1 ==> USART-TXD								
; PD2 ==> 										
; PD3 ==> 										
; PD4 ==> 										
; PD5 ==> LED GRN								
; PD6 ==> LED GELB								
; PD7 ==> LED ROT								
;												


; ATmega8 										
.include "m8def.inc"
; KONSTANTEN									
.equ TAKT							= 16000000	; 16 MHz	
; USART											
.equ BAUD							= 9600		; 38400 19200 9600
; SPI-CS										
.equ SPI_CS_PIN_NR					= 2
.equ SPI_CS_DDR						= DDRB
.equ SPI_CS_PORT					= PORTB
; SPI-MOSI										
.equ SPI_MOSI_PIN_NR				= 3
.equ SPI_MOSI_DDR					= DDRB
.equ SPI_MOSI_PORT		 			= PORTB
; SPI-MISO										
.equ SPI_MISO_PIN_NR				= 4
.equ SPI_MISO_DDR					= DDRB
.equ SPI_MISO_PORT		 			= PORTB
; SPI-SCK										
.equ SPI_SCK_PIN_NR					= 5
.equ SPI_SCK_DDR					= DDRB
.equ SPI_SCK_PORT		 			= PORTB
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR				= 5
.equ LED_GRUEN_DDR					= DDRD
.equ LED_GRUEN_PORT					= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR				= 6
.equ LED_GELB_DDR					= DDRD
.equ LED_GELB_PORT					= PORTD
; LED-ROT										
.equ LED_ROT_PIN_NR					= 7
.equ LED_ROT_DDR					= DDRD
.equ LED_ROT_PORT					= PORTD
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN						=SRAM_START+0		; 0x0060 START Adresse 				

.equ adr_SPI_DATENBEREICH			=SRAM_START+1

.equ adr_SRAM_CHECK					=SRAM_START+1001	; SRAM CHECK muss immer "S" sein	


; REGISTER										

.def SREG_KOPIE = R2
.def TEMP_KOPIE	= R3
.def SPI_DATA	= R4

.def NULL 	= R14
.def EINS 	= R15

.def temp 	= R16
.def temp1 	= R17
.def temp2 	= R18
.def temp3 	= R19
.def temp4 	= R20
.def temp5 	= R21
.def temp6 	= R22
.def temp7 	= R23
.def temp8 	= R24
.def temp9 	= R25





.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0

rjmp RESET      		; Reset Handler                           				
rjmp INTERRUPT_ERROR	;rjmp EXT_INT0   ; IRQ0 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   	  	; Timer1 Overflow Handler       
rjmp INTERRUPT_ERROR	;rjmp TIM0_OVF   ; Timer0 Overflow Handler           	
rjmp SPI_STC    		; SPI Transfer Complete Handler     					
rjmp INTERRUPT_ERROR	;rjmp ZEICHEN_EINLESEN 	; USART RX Complete Handler     
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI       ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Unkorrekten Interrupt Aufruf 						
INTERRUPT_ERROR:
	cbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB an	
	cbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  an	
	rcall wait_100ms
	sbi (LED_GELB_PORT),LED_GELB_PIN_NR			; GELB aus  
	sbi (LED_ROT_PORT),LED_ROT_PIN_NR			; ROT  aus	
	rcall wait_100ms
	rjmp INTERRUPT_ERROR

RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM lschen										
	rcall SRAM_CLEAR
; PORT B+C+D setzen									
	ldi temp, 0b00000000	; 
	out DDRB, temp
	out DDRC, temp
	out DDRD, temp
	out PORTB, temp				
	out PORTC, temp				
	out PORTD, temp				
; LED-INITIALISIERUNG								
	rcall LED_INITIALISIERUNG
; WDR_INITIALISIERUNG								
	rcall WDR_INITIALISIERUNG
; USART_INITIALISIEREN								
	rcall USART_INITIALISIEREN
; SPI-INITIALISIERUNG								
	rcall SPI_INITIALISIERUNG
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
	; Z-Zeiger (fr Datenspeicherung)
	ldi ZL, LOW (adr_SPI_DATENBEREICH)
	ldi ZH, HIGH(adr_SPI_DATENBEREICH)
; 	PROGRAMM_CHECK									
	rcall PROGRAMM_CHECK
; Interrupts freigeben								
	sei
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR											; Watch-Dog-RESET	
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an   	
	rcall WAIT_5ms
	sbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN aus   	
	rcall WAIT_50ms
	in temp,SPCR
	tst temp
	brne LOOP
;-------------------------------------------------------------------------------
; USART-AUSGABE																	
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an   	
	ldi temp1,'*'								; Vorspann		
	rcall USART_TX_TEMP1
	rcall USART_TX_TEMP1
	rcall USART_TX_TEMP1
	rcall USART_TX_DATA							; DATA			
LOOP1:
	rcall PROGRAMM_CHECK
	WDR
	cbi (LED_GRUEN_PORT),LED_GRUEN_PIN_NR		; GRN an   	
rjmp LOOP1
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "SPI_ANA_SPI.asm"				; SPI							
.include "SPI_ANA_USART.asm"
.include "SPI_ANA_SONSTIGES.asm"
.include "SPI_ANA_WAIT_16MHZ.asm"
; ##############################################################################
; ##############################################################################
; ##############################################################################





